"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.analyzeTimeSpent = exports.getUserContribution = exports.getCommitSummaries = void 0;
var git = require("./git");
var logger_1 = require("./logger");
function getCommitSummaries(config) {
    return __awaiter(this, void 0, void 0, function () {
        var repositories, countMerges, since, until, allCommits, commitsByEmail;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    repositories = config.repositories, countMerges = config.countMerges, since = config.since, until = config.until;
                    return [4 /*yield*/, git.getCommits({
                            gitPaths: repositories.map(function (r) { return (typeof r === 'string' ? r : r.path); }),
                            countMerges: countMerges,
                            since: since,
                            until: until,
                        })];
                case 1:
                    allCommits = _a.sent();
                    commitsByEmail = allCommits.reduce(function (map, commit) {
                        var email = commit.author.email || 'unknown';
                        if (config.emailAliases[email] !== undefined) {
                            email = config.emailAliases[email];
                        }
                        if (!map[email]) {
                            // eslint-disable-next-line no-param-reassign
                            map[email] = [];
                        }
                        map[email].push(commit);
                        return map;
                    }, {});
                    if (config.authors.length > 0) {
                        Object.keys(commitsByEmail).forEach(function (email) {
                            if (!config.authors.includes(email)) {
                                delete commitsByEmail[email];
                            }
                        });
                    }
                    Object.keys(commitsByEmail).forEach(function (email) {
                        var commits = commitsByEmail[email];
                        commitsByEmail[email] = {
                            commits: commits,
                        };
                    });
                    return [2 /*return*/, commitsByEmail];
            }
        });
    });
}
exports.getCommitSummaries = getCommitSummaries;
function getUserContribution(_a) {
    var _b, _c;
    var commits = _a.commits, firstCommitAdditionInMinutes = _a.firstCommitAdditionInMinutes, maxCommitDiffInMinutes = _a.maxCommitDiffInMinutes;
    if (commits.length === 0) {
        return {};
    }
    if (commits.length === 1) {
        var commit = commits[0];
        return _b = {},
            _b[commit.repo] = (_c = {},
                _c[asISOday(commit.date)] = {
                    hours: firstCommitAdditionInMinutes / 60,
                    commits: 1,
                },
                _c),
            _b;
    }
    var sortedCommits = commits.sort(oldestLastSorter);
    var repoSummary = {};
    var isoDaySet = new Set();
    var addCommitData = function (timeInMinutes, repository, date) {
        var isoDay = asISOday(date);
        isoDaySet.add(isoDay);
        if (!repoSummary[repository]) {
            repoSummary[repository] = {};
        }
        if (!repoSummary[repository][isoDay]) {
            repoSummary[repository][isoDay] = {
                commits: 0,
                hours: 0,
            };
        }
        repoSummary[repository][isoDay].commits += 1;
        repoSummary[repository][isoDay].hours += timeInMinutes / 60;
    };
    var lastTimeStamp = null;
    var numSessionsDetected = 0;
    var repoSet = new Set();
    sortedCommits.forEach(function (commit) {
        var diffInMinutes = lastTimeStamp && getDiffInMinutes(commit.date, lastTimeStamp);
        lastTimeStamp = commit.date;
        if (diffInMinutes === null || diffInMinutes > maxCommitDiffInMinutes) {
            var _a = getDateInfo(lastTimeStamp), date = _a.date, time = _a.time;
            var pauseLength = Math.round(diffInMinutes);
            numSessionsDetected += 1;
            logger_1.default.verbose(pauseLength + " minutes diff until " + date + " " + time, "\u2013 session starts.");
            diffInMinutes = firstCommitAdditionInMinutes;
        }
        repoSet.add(commit.repo);
        addCommitData(diffInMinutes, commit.repo, commit.date);
    });
    var author = sortedCommits[0].author;
    logger_1.default.verbose("\n " + author.name + " (" + author.email + ") had\n", numSessionsDetected + " sessions constructed using\n", sortedCommits.length + " commits, made in\n", repoSet.size + " repositories, done on\n", isoDaySet.size + " different days.\n");
    return repoSummary;
}
exports.getUserContribution = getUserContribution;
function analyzeTimeSpent(config) {
    return __awaiter(this, void 0, void 0, function () {
        var commitSummaries, firstCommitAdditionInMinutes, maxCommitDiffInMinutes, authorWorks, completeSummary;
        var _this = this;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, getCommitSummaries(config)];
                case 1:
                    commitSummaries = _a.sent();
                    firstCommitAdditionInMinutes = config.firstCommitAdditionInMinutes, maxCommitDiffInMinutes = config.maxCommitDiffInMinutes;
                    return [4 /*yield*/, Promise.all(Object.keys(commitSummaries).map(function (email) { return __awaiter(_this, void 0, void 0, function () {
                            var authorSummary, timeSummary;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        authorSummary = commitSummaries[email];
                                        return [4 /*yield*/, getUserContribution({
                                                commits: authorSummary.commits,
                                                firstCommitAdditionInMinutes: firstCommitAdditionInMinutes,
                                                maxCommitDiffInMinutes: maxCommitDiffInMinutes,
                                            })];
                                    case 1:
                                        timeSummary = _a.sent();
                                        return [2 /*return*/, { contributions: timeSummary, email: email }];
                                }
                            });
                        }); }))];
                case 2:
                    authorWorks = _a.sent();
                    completeSummary = {};
                    authorWorks.forEach(function (work) {
                        var email = work.email, contributions = work.contributions;
                        Object.keys(contributions).forEach(function (repository) {
                            if (!completeSummary[repository]) {
                                completeSummary[repository] = {};
                            }
                            completeSummary[repository][email] = contributions[repository];
                        });
                    });
                    return [2 /*return*/, completeSummary];
            }
        });
    });
}
exports.analyzeTimeSpent = analyzeTimeSpent;
var oldestLastSorter = function (a, b) {
    return a.date.getTime() - b.date.getTime();
};
var getDiffInMinutes = function (a, b) {
    return Math.abs(a.getTime() - b.getTime()) / 1000 / 60;
};
var getDateInfo = function (d) { return ({
    date: d.toISOString().substr(0, 10),
    time: d.toISOString().substr(11, 5),
}); };
var asISOday = function (d) { return d.toISOString().substr(0, 10); };
