"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseCommandLineArgs = void 0;
var program = require("commander");
var config_1 = require("../config");
var logger_1 = require("../logger");
function parseCommandLineArgs() {
    function int(val) {
        return parseInt(val, 10);
    }
    // eslint-disable-next-line @typescript-eslint/no-var-requires, global-require
    var programVersion = require('../../package.json').version;
    program
        .version(programVersion)
        .usage('[options]')
        .option('-a, --authors [email@gmail.com]', 'Only care about commits from these emails.' +
        wrapInDefault(config_1.defaultConfig.authors), parseArrayArg(','))
        .option('-d, --max-commit-diff [minutes]', 'max minutes between commits counted as one session.' +
        wrapInDefault(config_1.defaultConfig.maxCommitDiffInMinutes), int)
        .option('-f, --first-commit-add [minutes]', 'how many minutes first commit of session should add to total.' +
        wrapInDefault(config_1.defaultConfig.firstCommitAdditionInMinutes), int)
        .option('-s, --since [date]', 'Analyze data since date (including). \n' +
        '[today|lastweek|thismonth|yyyy-mm-dd]' +
        wrapInDefault(config_1.defaultConfig.since), String)
        .option('-u, --until [date]', 'Analyze data until date (excluding). \n' +
        '[today|lastweek|thismonth|yyyy-mm-dd]' +
        wrapInDefault(config_1.defaultConfig.until), String)
        .option('-r, --repositories [path,other-path]', 'Git repositories to analyze.' +
        wrapInDefault(config_1.defaultConfig.repositories.join(',')), parseArrayArg(','))
        .option('-e, --email [emailOther=emailMain]', 'Group person by email.', parseDictArg(',', '='))
        .option('-m, --merge-request [false|true]', 'Include merge requests into calculation.' +
        wrapInDefault(config_1.defaultConfig.countMerges), parseBooleanArg)
        .option('-i, --ignore-timesheetrc', 'Ignores .timesheetrc from home directory.' +
        wrapInDefault(config_1.defaultConfig.ignoreConfigFile), parseArgTrueIfSpecified)
        .option('-j, --json', 'Reports in JSON format.' + wrapInDefault(config_1.defaultConfig.json), parseArgTrueIfSpecified)
        .option('-v --verbose', 'Prints extra stats' + wrapInDefault(config_1.defaultConfig.verbose), parseArgTrueIfSpecified)
        .option('-D --debug', 'Prints debug information' + wrapInDefault(config_1.defaultConfig.debug), parseArgTrueIfSpecified);
    program.on('--help', function () {
        logger_1.default.output("\nExamples:\n\n  - Estimate hours of project\n\n   $ timesheet\n\n  - Estimate hours by me@example.com\n\n   $ timesheet -a me@example.com\n\n  - Estimate hours where developers commit seldom\n\n   $ timesheet --max-commit-diff 240\n\n  - Estimate hours in when working 5 hours before first commit of day\n\n   $ timesheet --first-commit-add 300\n\n  - Estimate hours work this month\n\n   $ timesheet --since thismonth\n\n  - Estimate hours work until 2020-01-01\n\n   $ timesheet --until 2020-01-01\n\n  For more details, visit https://github.com/tomfa/git-csv-timesheet\n  ");
    });
    program.parse(process.argv);
    var confArgs = {
        maxCommitDiffInMinutes: program.maxCommitDiff,
        firstCommitAdditionInMinutes: program.firstCommitAdd,
        since: program.since,
        until: program.until,
        repositories: program.repositories,
        countMerges: program.countMerges,
        emailAliases: program.email,
        ignoreConfigFile: program.ignoreTimesheetrc,
        authors: program.authors,
        json: program.json,
        verbose: program.verbose,
        debug: program.debug,
    };
    Object.entries(confArgs).forEach(function (_a) {
        var key = _a[0], value = _a[1];
        if (value === undefined) {
            delete confArgs[key];
        }
    });
    return confArgs;
}
exports.parseCommandLineArgs = parseCommandLineArgs;
var parseBooleanArg = function (value) {
    return value ? value.trim() === 'true' : undefined;
};
var parseArrayArg = function (separator) { return function (value) {
    return value.trim() ? value.split(',').map(function (v) { return v.trim(); }) : undefined;
}; };
var parseDictArg = function (separator, keyValueSeparator) { return function (argumentValue) {
    if (!argumentValue) {
        return undefined;
    }
    var map = argumentValue.split(separator).reduce(function (aliasMap, singleAlias) {
        var _a;
        var _b = singleAlias.split(keyValueSeparator), key = _b[0], value = _b[1];
        if (!value) {
            logger_1.default.error("Argument " + argumentValue + " is invalid.", "Part \"" + singleAlias + "\" is missing a \"" + keyValueSeparator + "\".");
            return aliasMap;
        }
        return __assign(__assign({}, aliasMap), (_a = {}, _a[key] = value, _a));
    }, {});
    if (Object.keys(map).length === 0) {
        return undefined;
    }
    return map;
}; };
var parseArgTrueIfSpecified = function () { return true; };
var wrapInDefault = function (value) {
    if (value === undefined) {
        return '';
    }
    if (value instanceof Array) {
        if (value.length === 0) {
            return '';
        }
        // eslint-disable-next-line no-param-reassign
        value = value.join(',');
    }
    return "\n[default: " + value + "]";
};
