"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConfig = exports.defaultConfig = exports.HOMEDIR_CONFIG_FILE_NAME = void 0;
var moment = require("moment");
var files_1 = require("./files");
var DATE_FORMAT = 'YYYY-MM-DD';
exports.HOMEDIR_CONFIG_FILE_NAME = '.timesheetrc';
exports.defaultConfig = {
    // Maximum time diff between 2 subsequent commits in minutes which are
    // counted to be in the same coding "session"
    maxCommitDiffInMinutes: 3 * 60,
    // How many minutes should be added for the first commit of coding session
    firstCommitAdditionInMinutes: 60,
    // Include commits since time x
    since: 'always',
    until: 'always',
    // Include merge requests
    countMerges: true,
    // Git repo
    repositories: ['.'],
    // Aliases of emails for grouping the same activity as one person
    emailAliases: {},
    // Filters authors from output
    authors: [],
    // Ignores .timesheetrc config files
    ignoreConfigFile: false,
    // Logs stats found under parsing
    verbose: false,
    // Report with JSON instead of csv
    json: false,
    // Logs debug information
    debug: false,
};
function parseInputDate(inputDate) {
    if (inputDate instanceof Date) {
        return inputDate;
    }
    switch (inputDate) {
        case 'today':
            return moment().startOf('day').toDate();
        case 'yesterday':
            return moment().startOf('day').subtract(1, 'day').toDate();
        case 'thisweek':
            return moment().startOf('week').toDate();
        case 'lastweek':
            return moment().startOf('week').subtract(1, 'week').toDate();
        case 'thismonth':
            return moment().startOf('month').toDate();
        case 'lastmonth':
            return moment().startOf('month').subtract(1, 'month').toDate();
        case 'always':
            return 'always';
        default:
            // XXX: Moment tries to parse anything, results might be weird
            return moment(inputDate, DATE_FORMAT).toDate();
    }
}
function getHomeDirectoryConfig() {
    try {
        return {
            fileConfig: files_1.readHomeDirectoryConfig(exports.HOMEDIR_CONFIG_FILE_NAME),
            path: "~/" + exports.HOMEDIR_CONFIG_FILE_NAME + "}",
        };
    }
    catch (error) {
        return { fileConfig: {}, path: null };
    }
}
function getConfig(overrides) {
    var ignoreConfigFile = overrides.ignoreConfigFile === true ||
        (overrides.ignoreConfigFile === undefined &&
            exports.defaultConfig.ignoreConfigFile === true);
    var _a = (!ignoreConfigFile && getHomeDirectoryConfig()) || {}, fileConfig = _a.fileConfig, path = _a.path;
    var config = __assign(__assign(__assign({}, exports.defaultConfig), (fileConfig || {})), overrides);
    config.since = parseInputDate(config.since);
    config.until = parseInputDate(config.until);
    // TODO: Verify that each gitPath is a valid repository
    return { config: config, configFilePath: path };
}
exports.getConfig = getConfig;
