"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBranchCommits = exports.getBranchLatestCommit = exports.getCommitsForRepository = exports.getCommits = exports.isShallowGitRepo = void 0;
var fs = require("fs");
var git = require("nodegit");
var moment = require("moment");
var logger_1 = require("./logger");
function isShallowGitRepo(path) {
    return fs.existsSync(path + '.git/shallow');
}
exports.isShallowGitRepo = isShallowGitRepo;
function getCommits(_a) {
    var gitPaths = _a.gitPaths, countMerges = _a.countMerges, since = _a.since, until = _a.until;
    return __awaiter(this, void 0, void 0, function () {
        var listOfCommitLists;
        var _this = this;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, Promise.all(gitPaths.map(function (path) { return __awaiter(_this, void 0, void 0, function () {
                        return __generator(this, function (_a) {
                            return [2 /*return*/, getCommitsForRepository({
                                    gitPath: path,
                                    countMerges: countMerges,
                                    since: since,
                                    until: until,
                                })];
                        });
                    }); }))];
                case 1:
                    listOfCommitLists = _b.sent();
                    return [2 /*return*/, listOfCommitLists.reduce(function (flattenedList, currentList) { return __spreadArrays(flattenedList, currentList); }, [])];
            }
        });
    });
}
exports.getCommits = getCommits;
function getCommitsForRepository(_a) {
    var gitPath = _a.gitPath, countMerges = _a.countMerges, since = _a.since, until = _a.until;
    return __awaiter(this, void 0, void 0, function () {
        var repository, allReferences, references, allCommits, latestBranchCommits, branchCommits, uniqueCommits, isNotMergeCommit;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    if (isShallowGitRepo(gitPath)) {
                        logger_1.default.warn("Cannot analyze shallow git repo: " + gitPath + "!");
                        logger_1.default.warn("To fix this issue: run git fetch --unshallow inside " + gitPath);
                        return [2 /*return*/, []];
                    }
                    return [4 /*yield*/, git.Repository.open(gitPath)];
                case 1:
                    repository = _b.sent();
                    return [4 /*yield*/, getAllReferences(repository)];
                case 2:
                    allReferences = _b.sent();
                    references = allReferences.filter(function (r) { return r.match(/refs\/heads\/.*/); });
                    allCommits = [];
                    return [4 /*yield*/, Promise.all(references.map(function (branchName) {
                            return getBranchLatestCommit(repository, branchName);
                        }))];
                case 3:
                    latestBranchCommits = _b.sent();
                    return [4 /*yield*/, Promise.all(latestBranchCommits.map(function (latestCommit) {
                            return getBranchCommits({ latestCommit: latestCommit, since: since, until: until, repository: repository });
                        }))];
                case 4:
                    branchCommits = _b.sent();
                    branchCommits.forEach(function (commitList) {
                        return commitList.forEach(function (commit) { return allCommits.push(commit); });
                    });
                    uniqueCommits = allCommits.filter(generateUniquesBasedOnKeyFilter('sha'));
                    if (countMerges) {
                        return [2 /*return*/, uniqueCommits];
                    }
                    isNotMergeCommit = function (commit) { return !commit.message.startsWith('Merge '); };
                    return [2 /*return*/, uniqueCommits.filter(isNotMergeCommit)];
            }
        });
    });
}
exports.getCommitsForRepository = getCommitsForRepository;
function getAllReferences(repo) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    return repo.getReferenceNames(git.Reference.TYPE.ALL);
}
function getBranchLatestCommit(repo, branchName) {
    return __awaiter(this, void 0, void 0, function () {
        var branch;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, repo.getBranch(branchName)];
                case 1:
                    branch = _a.sent();
                    return [2 /*return*/, repo.getBranchCommit(branch.name())];
            }
        });
    });
}
exports.getBranchLatestCommit = getBranchLatestCommit;
function getBranchCommits(_a) {
    var latestCommit = _a.latestCommit, repository = _a.repository, since = _a.since, until = _a.until;
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_b) {
            if (!repository) {
                // eslint-disable-next-line no-param-reassign
                repository = latestCommit.owner();
            }
            return [2 /*return*/, new Promise(function (resolve, reject) {
                    var history = latestCommit.history();
                    var commits = [];
                    history.on('commit', function (commit) {
                        var commitAuthor = commit.author();
                        var author = commitAuthor
                            ? { name: commitAuthor.name(), email: commitAuthor.email() }
                            : { name: 'unknown', email: 'unknown@example.com' };
                        var commitData = {
                            sha: commit.sha(),
                            date: commit.date(),
                            message: commit.message(),
                            author: author,
                            repo: repository.commondir().replace('/.git/', ''),
                        };
                        var commitDate = moment(commitData.date.toISOString());
                        var sinceAlways = since === 'always' || !since;
                        var isValidSince = sinceAlways || commitDate.isAfter(since);
                        var untilAlways = until === 'always' || !until;
                        var isValidUntil = untilAlways || commitDate.isBefore(until);
                        if (isValidSince && isValidUntil) {
                            commits.push(commitData);
                        }
                    });
                    history.on('end', function () { return resolve(commits); });
                    history.on('error', function (err) { return reject(err); });
                    history.start();
                })];
        });
    });
}
exports.getBranchCommits = getBranchCommits;
var generateUniquesBasedOnKeyFilter = function (key) {
    var uniques = new Set();
    return function (item) {
        var alreadyAdded = uniques.has(item[key]);
        if (!alreadyAdded) {
            uniques.add(item[key]);
        }
        return !alreadyAdded;
    };
};
