"use strict";
/* eslint-disable no-param-reassign */
Object.defineProperty(exports, "__esModule", { value: true });
exports.printAsJSON = exports.printAsCSV = void 0;
var logger_1 = require("./logger");
var CSV_SEPARATOR = ';';
var DECIMAL_POINTS_PRECISION = 1;
var PrintColumn;
(function (PrintColumn) {
    PrintColumn["AUTHOR"] = "author";
    PrintColumn["DATE"] = "date";
    PrintColumn["PROJECT"] = "project";
    PrintColumn["REPOSITORY"] = "repository";
    PrintColumn["TASK"] = "task";
    PrintColumn["HOURS"] = "hours";
})(PrintColumn || (PrintColumn = {}));
var flattenSummary = function (summary) {
    var flatSummary = [];
    Object.keys(summary).forEach(function (repository) {
        var repositorySummary = summary[repository];
        Object.keys(repositorySummary).forEach(function (author) {
            var authorSummary = repositorySummary[author];
            Object.keys(authorSummary).forEach(function (date) {
                var hours = authorSummary[date].hours;
                flatSummary.push({ date: date, hours: hours, author: author, repository: repository });
            });
        });
    });
    return flatSummary.sort(getSortByKeysFilter(PrintColumn.AUTHOR, PrintColumn.DATE, PrintColumn.REPOSITORY));
};
var DEFAULT_PRINT_LINES = [
    PrintColumn.AUTHOR,
    PrintColumn.DATE,
    // PrintColumn.PROJECT,
    PrintColumn.REPOSITORY,
    // PrintColumn.TASK,
    PrintColumn.HOURS,
];
exports.printAsCSV = function (_a) {
    var summary = _a.summary, _b = _a.columns, columns = _b === void 0 ? undefined : _b;
    if (columns) {
        throw Error('columns param not yet supported');
    }
    // eslint-disable-next-line no-param-reassign
    columns = DEFAULT_PRINT_LINES;
    if (getRepositories(summary).length === 1) {
        columns = columns.filter(function (c) { return c !== PrintColumn.REPOSITORY; });
    }
    if (getAuthors(summary).length === 1) {
        columns = columns.filter(function (c) { return c !== PrintColumn.AUTHOR; });
    }
    var flatSummary = flattenSummary(summary).sort();
    logger_1.default.output(columns.join(CSV_SEPARATOR));
    flatSummary.forEach(function (summaryItem) {
        var printValues = [];
        columns.forEach(function (column) { return printValues.push(summaryItem[column]); });
        logger_1.default.output(printValues.map(printColumnsString).join(CSV_SEPARATOR));
    });
    var totalHoursSpent = flatSummary.reduce(function (timespent, item) { return timespent + item.hours; }, 0);
    logger_1.default.verbose("\nTotal hours: " + totalHoursSpent.toFixed(1));
};
exports.printAsJSON = function (_a) {
    var summary = _a.summary, columns = _a.columns;
    logger_1.default.output(JSON.stringify(summary, undefined, 2));
};
var getSortByKeysFilter = function () {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return function (a, b) {
        // eslint-disable-next-line no-restricted-syntax
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            if (a[key] > b[key]) {
                return 1;
            }
            if (a[key] < b[key]) {
                return -1;
            }
        }
        return 0;
    };
};
var getRepositories = function (summary) {
    return Object.keys(summary);
};
var getAuthors = function (summary) {
    var authorSet = Object.values(summary).reduce(function (authors, authorSummary) {
        Object.keys(authorSummary).forEach(function (author) { return authors.add(author); });
        return authors;
    }, new Set());
    return Array.from(authorSet);
};
var printColumnsString = function (value) {
    if (typeof value === 'string') {
        return value;
    }
    if (typeof value === 'number') {
        return value.toFixed(DECIMAL_POINTS_PRECISION);
    }
    return String(value);
};
