export declare type EmailAliases = {
    [email: string]: string;
};
export declare type RepoAuthorContribution = {
    [isoDateDay: string]: {
        hours: number;
        commits: number;
    };
};
export declare type CommitSummary = {
    commits: Commit[];
};
export declare type RepoWorkSummary = {
    [email: string]: RepoAuthorContribution;
};
export declare type CompleteSummary = {
    [repository: string]: RepoWorkSummary;
};
export declare type CompleteUserSummary = {
    contributions: {
        [repository: string]: RepoAuthorContribution;
    };
    email: string;
};
export declare type Commit = {
    sha: string;
    date: Date;
    message: string;
    author: {
        name: string;
        email: string;
    };
    repo: string;
};
export declare type RepositoryConfig = {
    project: string;
    path: string;
    countMerges: boolean;
    trackTasks: boolean;
};
export declare type Config = {
    maxCommitDiffInMinutes: number;
    firstCommitAdditionInMinutes: number;
    since: string | Date;
    until: string | Date;
    countMerges: boolean;
    repositories: Array<string | RepositoryConfig>;
    authors: Array<string>;
    emailAliases: EmailAliases;
    ignoreConfigFile: boolean;
    verbose: boolean;
    json: boolean;
    debug: boolean;
};
export declare type HomeDirectoryConfig = {
    maxCommitDiffInMinutes: number;
    firstCommitAdditionInMinutes: number;
    countMerges: boolean;
    emailAliases: EmailAliases;
    branch: string | null;
    authors: Array<string>;
    repositories: Array<string | RepositoryConfig>;
};
